/**
 * Use of the WebDLPClient and related source code is subject to the
 * terms of the following license:
 *
 * Copyright (c) 2013 Carnegie Mellon University. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following acknowledgments and disclaimers.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. All advertising materials for third-party software mentioning features or
 * use of this software must display the following disclaimer:
 *
 * “Neither Carnegie Mellon University nor its Software Engineering Institute
 * have reviewed or endorsed this software”
 *
 * 4. The names “Carnegie Mellon University,” "CERT” and/or “Software
 * Engineering Institute" shall not be used to endorse or promote products
 * derived from this software without prior written permission. For written
 * permission, please contact permission@sei.cmu.edu.
 *
 * 5. Products derived from this software may not be called "CERT" nor may
 * "CERT" appear in their names without prior written permission of
 * permission@sei.cmu.edu.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 * acknowledgment:
 *
 * "This product includes software developed by CERT with funding and support
 * from the US Government under Contract No. FA8721-05-C-0003. The U.S.
 * Government's rights to use, modify, reproduce, release, perform, display, or
 * disclose this material are restricted by the Rights in Technical
 * Data-Noncommercial Items clauses (DFAR 252-227.7013 and DFAR 252-227.7013
 * Alternate I contained in the foregoing identified contract.
 *
 * THIS SOFTWARE IS PROVIDED BY CARNEGIE MELLON UNIVERSITY ``AS IS'' AND
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER EXPRESS OR
 * IMPLIED, AS TO ANY MATTER, AND ALL SUCH WARRANTIES, INCLUDING WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, ARE EXPRESSLY
 * DISCLAIMED. WITHOUT LIMITING THE GENERALITY OF THE FOREGOING, CARNEGIE MELLON
 * UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND RELATING TO EXCLUSIVITY,
 * INFORMATIONAL CONTENT, ERROR-FREE OPERATION, RESULTS TO BE OBTAINED FROM USE,
 * FREEDOM FROM PATENT, TRADEMARK AND COPYRIGHT INFRINGEMENT AND/OR FREEDOM FROM
 * THEFT OF TRADE SECRETS.”
 *
 */
package webdlpclient;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.HashMap;

/**
 * This class is a simple API for communicating with a WebDLPIndexer instance
 *
 * @author Todd Lewellen
 */
public class WebDLPClient {

    /**
     * The default port on which to connect to the WebDLPIndexer instance
     */
    public final static int DEFAULT_PORT = 22345;
    private String logFileName = null;
    private boolean log = false;
    private int port;
    private InetAddress ipAddress;
    private Socket socket;
    private ObjectOutputStream writer;
    private ObjectInputStream reader;

    /**
     * @param ipAddress The IP address of the WebDLPIndexer instance
     * @param portNum The port to connect to
     * @throws UnknownHostException
     */
    public WebDLPClient(String ipAddress, int portNum, String logFileName) throws UnknownHostException {
        initializeLogger(logFileName);
        port = portNum;
        this.ipAddress = InetAddress.getByName(ipAddress);
    }

    /**
     * Uses the default port (22345) and localhost when trying to connect to the
     * WebDLPIndexer instance
     */
    public WebDLPClient(String logFileName) {
        initializeLogger(logFileName);
        port = DEFAULT_PORT;
        try {
            ipAddress = InetAddress.getLocalHost();
        } catch (UnknownHostException ex) {
            if(log)
                Logger.exception(ex);   
            else
                ex.printStackTrace();
        }
    }

    /**
     * This takes input text and sends it to the WebDLPIndexer instance, which
     * responds with a HashMap of all of the filenames in the index and their
     * percentage match score against the input text
     *
     * @param text The text to compare against the Apache Lucene index
     * @return A HashMap of filenames and their percentage match similarity
     * score against the input text
     * @throws IOException
     */
    public HashMap<String, Double> testSimilarity(String text) throws IOException {

        openWriter();
        writer.writeUTF(text);
        writer.writeInt(-1);
        writer.flush();

        reader = new ObjectInputStream(socket.getInputStream());
        HashMap<String, Double> map = null;
        try {
            map = (HashMap<String, Double>) reader.readObject();            
        } catch (ClassNotFoundException ex) {
            if(log)
                Logger.exception(ex);
            else
                ex.printStackTrace();
        }

        closeWriter();
        return map;
    }

    /**
     * This takes input text and sends it to the WebDLPIndexer instance, which
     * responds with a HashMap of all of the filenames in the index and their
     * percentage match score against the input text
     *
     * @param text The text to compare against the Apache Lucene index
     * @param threshold The threshold (between 0 and 100) by which to compare
     * @return A HashMap of any filenames (and their percentage match similarity
     * score against the input text) where the score was above the threshold
     * @throws IOException
     */
    public HashMap<String, Double> testSimilarityWithThreshold(String text, int threshold) throws IOException {

        openWriter();
        writer.writeUTF(text);
        writer.flush();
        writer.writeInt(threshold);
        writer.flush();

        reader = new ObjectInputStream(socket.getInputStream());
        HashMap<String, Double> map = null;
        try {
            map = (HashMap<String, Double>) reader.readObject();
        } catch (ClassNotFoundException ex) {
            if(log)
                Logger.exception(ex);
            else
                ex.printStackTrace();
        }

        closeWriter();
        return map;
    }

    /**
     * Simply tests to see if the input text scores above the threshold when
     * compared against any files in the index. If a match is found above the
     * threshold, this method will return false, as in "The text is not
     * acceptable for outbound traffic"
     *
     * @param text The text to compare against the Apache Lucene index
     * @param threshold The threshold (between 0 and 100) by which to compare
     * @return A HashMap of any filenames (and their percentage match similarity
     * score against the input text) where the score was above the threshold
     * @throws IOException
     */
    public boolean testIfAcceptable(String text, int threshold) throws IOException {
        HashMap<String, Double> map = testSimilarityWithThreshold(text, threshold);
        return (map.size() > 0 ? true : false);
    }

    private void openWriter() {
        try {
            socket = new Socket(ipAddress, port);
            writer = new ObjectOutputStream(socket.getOutputStream());
        } catch (IOException ex) {
            if(log)
                Logger.exception(ex);
            else
                ex.printStackTrace();
        }
    }

    private void closeWriter() {
        try {
            writer.close();
            reader.close();
            socket.close();
        } catch (IOException ex) {
            if(log)
                Logger.exception(ex);
            else
                ex.printStackTrace();
        }
    }

    private void initializeLogger(String logFileName) {
        if (logFileName != null) {
            this.logFileName = logFileName;
            log = true;
            //Initialize the Logger        
            Logger.logFile = new File(logFileName);
        }
    }
}
